/* Emulation of the Z80 CPU with hooks into the other parts of z81.
 * Copyright (C) 1994 Ian Collier.
 * z81 changes (C) 1995-2001 Russell Marks.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>	/* for memset/memcpy */
#include "common.h"
#include "sound.h"
#include "z80.h"

#include <stdio.h>/* for FILE access Z81 FILES */

#ifdef Win32
#include <windows.h>
//BOOL Ask_Z81(BOOL bSave);
#endif

#define parity(a) (partable[a])

unsigned char partable[256]={
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      0, 4, 4, 0, 4, 0, 0, 4, 4, 0, 0, 4, 0, 4, 4, 0,
      4, 0, 0, 4, 0, 4, 4, 0, 0, 4, 4, 0, 4, 0, 0, 4
   };


unsigned long tstates=0,tsmax=65000,frames=0;
unsigned long tsmaxNTSC=65000;

/* odd place to have this, but the display does work in an odd way :-) */
unsigned char scrnbmp_new[ZX_VID_FULLWIDTH*ZX_VID_FULLHEIGHT/8]; /* written */
unsigned char scrnbmp[ZX_VID_FULLWIDTH*ZX_VID_FULLHEIGHT/8];	/* displayed */
unsigned char scrnbmp_old[ZX_VID_FULLWIDTH*ZX_VID_FULLHEIGHT/8];
						/* checked against for diffs */
static int liney=0;
static int vsy=0;
static unsigned long linestart=0;
static int vsync_toggle=0,vsync_lasttoggle=0;

// Public declaration : Z80 Registers.
unsigned char a, f, b, c, d, e, h, l;
unsigned char r, a1, f1, b1, c1, d1, e1, h1, l1, i, iff1, iff2, im;
unsigned short pc;
unsigned short ix, iy, sp;
unsigned long nextlinetime=0,linegap=208,lastvsyncpend=0;
unsigned char radjust;
unsigned char ixoriy, new_ixoriy;
unsigned char intsample=0;
unsigned char op;
int ulacharline=0;
int nmipend=0,intpend=0,vsyncpend=0,vsynclen=0;
int hsyncskip=0;
int framewait=0;


//


int ay_reg=0;


#define LINEX 	((tstates-linestart)>>2)


/* for vsync off -> on */
void vsync_raise(void)
{
/* save current pos */
vsy=liney;
}


/* for vsync on -> off */
void vsync_lower(void)
{
int ny=liney,y;

vsync_toggle++;

/* we don't emulate this stuff by default; if nothing else,
 * it can be fscking annoying when you're typing in a program.
 */
if(!vsync_visuals)
  return;

/* even when we do emulate it, we don't bother with x timing,
 * just the y. It gives reasonable results without being too
 * complicated, I think.
 */
 // Forth NTSC stuff.
if(vsy<0) vsy=0;
if(vsy>=ZX_VID_FULLHEIGHT) vsy=ZX_VID_FULLHEIGHT-1;
if(ny<0) ny=0;
if(ny>=ZX_VID_FULLHEIGHT) ny=ZX_VID_FULLHEIGHT-1;

/* XXX both of these could/should be made into single memset calls */
if(ny<vsy)
  {
  /* must be wrapping around a frame edge; do bottom half */
  for(y=vsy;y<ZX_VID_FULLHEIGHT;y++)
    memset(scrnbmp_new+y*(ZX_VID_FULLWIDTH/8),0xff,ZX_VID_FULLWIDTH/8);
  vsy=0;
  }

for(y=vsy;y<ny;y++)
  memset(scrnbmp_new+y*(ZX_VID_FULLWIDTH/8),0xff,ZX_VID_FULLWIDTH/8);
}


void mainloop()
{
/********************************************************************
Move to publics declaration to retrive them in the Z81 functions.
unsigned char a, f, b, c, d, e, h, l;
unsigned char r, a1, f1, b1, c1, d1, e1, h1, l1, i, iff1, iff2, im;
unsigned short pc;
unsigned short ix, iy, sp;
unsigned long nextlinetime=0,linegap=208,lastvsyncpend=0;
unsigned char radjust;
unsigned char ixoriy, new_ixoriy;
unsigned char intsample=0;
unsigned char op;
int ulacharline=0;
int nmipend=0,intpend=0,vsyncpend=0,vsynclen=0;
int hsyncskip=0;
int framewait=0;
********************************************************************/

a=f=b=c=d=e=h=l=a1=f1=b1=c1=d1=e1=h1=l1=i=iff1=iff2=im=r=0;
ixoriy=new_ixoriy=0;
ix=iy=sp=pc=0;
tstates=radjust=0;
nextlinetime=linegap;

if(autoload)
  {
            

// -> Load a P file.            
  /* we load a snapshot, in effect. The memory was done by
   * common.c, this does the registers.
   */
  static unsigned char bit1[9]={0xFF,0x80,0xFC,0x7F,0x00,0x80,0x00,0xFE,0xFF};
  static unsigned char bit2[4]={0x76,0x06,0x00,0x3e};
  
  /* memory will already be zeroed at this point */
  memcpy(mem+0x4000,bit1,9);
  memcpy(mem+0x7ffc,bit2,4);
  a=0x0B; f=0x85; b=0x00; c=0xFF;
  d=0x43; e=0x99; h=0xC3; l=0x99;
  a1=0xE2; f1=0xA1; b1=0x81; c1=0x02;
  d1=0x00; e1=0x2B; h1=0x00; l1=0x00;
  i=0x1E; iff1=iff2=0;
  im=2;
  r=0xDD; radjust=0xCA;
  ix=0x281; iy=0x4000;
  sp=0x7FFC;
  pc=0x207;

  /* finally, load. It'll reset (via reset81) if it fails. */
  load_p(32768);
  
 // <- Load a P file.

  /* wait for a real frame, to avoid an annoying frame `jump'. */
  framewait=1;
  }

while(1)
  {
  /* this *has* to be checked before radjust is incr'd */
  if(intsample && !(radjust&64))
    intpend=1;
  
  ixoriy=new_ixoriy;
  new_ixoriy=0;
  intsample=1;
  op=fetch(pc&0x7fff);
  if((pc&0x8000) && !(op&64))
    {
    int x,y,v;
    
    /* do the ULA's char-generating stuff */
    x=LINEX;
    y=liney;
/*    printf("ULA %3d,%3d = %02X\n",x,y,op);*/
    if(y>=0 && y<ZX_VID_FULLHEIGHT && x>=0 && x<ZX_VID_FULLWIDTH/8)
      {
      /* XXX I think this is what's needed for the `true hi-res'
       * stuff from the ULA's side, but the timing is messed up
       * at the moment so not worth it currently.
       */
#if 0
      if((tstates-linestart)==3)
        v=mem[((i&0xfe)<<8)|radjust];
      else
#endif
        v=mem[((i&0xfe)<<8)|((op&63)<<3)|ulacharline];
      if(taguladisp) v^=128;
      scrnbmp_new[y*(ZX_VID_FULLWIDTH/8)+x]=((op&128)?~v:v);
      }
    
    op=0;	/* the CPU sees a nop */
    }

  pc++;
  radjust++;
  
  switch(op)
    {
#include "z80ops.c"
    }
  
  if(tstates>=tsmax)
    {
    tstates-=tsmax;
    linestart-=tsmax;
    nextlinetime-=tsmax;
    lastvsyncpend-=tsmax;
    vsync_lasttoggle=vsync_toggle;
    vsync_toggle=0;
    
    frames++;
    frame_pause();
    }

  /* the vsync length test is pretty arbitrary, because
   * the timing isn't very accurate (more or less an instruction
   * count) - but it's good enough in practice.
   *
   * the vsync_toggle test is fairly arbitrary too;
   * there has to have been `not too many' for a TV to get
   * confused. In practice, more than one would screw it up,
   * but since we could be looking at over a frames' worth
   * given where vsync_toggle is zeroed, we play it safe.
   * also, we use a copy of the previous chunk's worth,
   * since we need a full frame(-plus) to decide this.
   */
  if(vsynclen && !vsync)
    {
    if(vsynclen>=10)
      {
      if(vsync_lasttoggle<=2)
        {
        vsyncpend=1;	/* start of frame */
        /* FAST mode screws up without this, but it's a bit
         * unpleasant... :-/
         */
        tstates=nextlinetime;
        }
      }
    else
      {
      /* independent timing for this would be awkward, so
       * anywhere on the line is good enough. Also,
       * don't count it as a toggle.
       */
      vsync_toggle--;
      hsyncskip=1;
      }
    }

  /* should do this all the time vsync is set */
  if(vsync)
    {
    ulacharline=0;
    vsynclen++;
    }
  else
    vsynclen=0;
  
  if(tstates>=nextlinetime)	/* new line */
    {
    /* generate fake sync if we haven't had one for a while;
     * but if we just loaded/saved, wait for the first real frame instead
     * to avoid jumpiness.
     */
    if(!vsync && tstates-lastvsyncpend>=tsmax && !framewait)
      vsyncpend=1;

    /* but that won't ever happen if we always have vsync on -
     * i.e., if we're grinding away in FAST mode. So for that
     * case, we check for vsync being held for a full frame.
     */
    if(vsync_visuals && vsynclen>=tsmax)
      {
      vsyncpend=1;
      vsynclen=1;
      
      memset(scrnbmp,0xff,sizeof(scrnbmp));	/* blank the screen */
      goto postcopy;				/* skip the usual copying */
      }

    if(!vsyncpend)
      {
      liney++;
      
      if(hsyncgen && !hsyncskip)
        {
/*        printf("hsync %d\n",tstates);*/
        ulacharline++;
        ulacharline&=7;
        }
      }
    else
      {
      memcpy(scrnbmp,scrnbmp_new,sizeof(scrnbmp));
      postcopy:
      memset(scrnbmp_new,0,sizeof(scrnbmp_new));
      lastvsyncpend=tstates;
      vsyncpend=0;
      framewait=0;
      liney=-1;		/* XXX might be something up here */
      
/*      printf("FRAME START - %s\n",(mem[16443]&128)?"slow":"fast");*/
      }
    
    if(nmigen)
      nmipend=1;

    hsyncskip=0;
    linestart=nextlinetime;
    nextlinetime+=linegap;
    }
  
  if(intsample && nmipend)
    {
    nmipend=0;

    if(nmigen)
      {
/*      printf("NMI line %d tst %d\n",liney,tstates);*/
      iff2=iff1;
      iff1=0;
      /* hardware syncs tstates to falling of NMI pulse (?),
       * so a slight kludge here...
       */
      if(fetch(pc&0x7fff)==0x76)
        {
        pc++;
        tstates=linestart;
        }
      else
        {
        /* this seems curiously long, but equally, seems
         * to be just about right. :-)
         */
        tstates+=27;
        }
      push2(pc);
      pc=0x66;
      }
    }

  if(intsample && intpend)
    {
    intpend=0;

    if(iff1)
      {
/*      printf("int line %d tst %d\n",liney,tstates);*/
      if(fetch(pc&0x7fff)==0x76)pc++;
      iff1=iff2=0;
      tstates+=5; /* accompanied by an input from the data bus */
      switch(im)
        {
        case 0: /* IM 0 */
        case 1: /* undocumented */
        case 2: /* IM 1 */
          /* there is little to distinguish between these cases */
          tstates+=9; /* perhaps */
          push2(pc);
          pc=0x38;
          break;
        case 3: /* IM 2 */
          /* (seems unlikely it'd ever be used on the '81, but...) */
          tstates+=13; /* perhaps */
          {
          int addr=fetch2((i<<8)|0xff);
          push2(pc);
          pc=addr;
          }
        }
      }
    }

  /* this isn't used for any sort of Z80 interrupts,
   * purely for the emulator's UI.
   */
  if(interrupted)
    {
    if(interrupted==1)
      {
      do_interrupt();	/* also zeroes it */
      }
    else	/* must be 2 */
      {
      /* a kludge to let us do a reset */
      interrupted=0;
      a=f=b=c=d=e=h=l=a1=f1=b1=c1=d1=e1=h1=l1=i=iff1=iff2=im=r=0;
      ixoriy=new_ixoriy=0;
      ix=iy=sp=pc=0;
      tstates=radjust=0;
      nextlinetime=linegap;
      vsyncpend=vsynclen=0;
      hsyncskip=0;
      }
    }
  }
}

  
/***************************************************************************
 * Read a Z81 File                                                         *
 ***************************************************************************/

void *Load_Z81(char *filename) {
	char line[255], key[64],datas[65536];
	int count, index, line_count;
	
	static unsigned long memorylow;
	static unsigned long HL;   // = (h<<8)+l
    static unsigned long HL1;  // = (h1<<8)+l1
    static unsigned long DE;   // = (d<<8)+e
    static unsigned long DE1;  // = (d1<<8)+e1
    static unsigned long BC;   // = (b<<8)+c
    static unsigned long BC1;  // = (b1<<8)+c1
    static unsigned long AF;   // = (a<<8)+f
    static unsigned long AF1;  // = (a1<<8)+f1
    static unsigned long IR;   // = (i<<8)+r
    static int scard,decay;
    
      framewait=0;    
      interrupted=0;
      a=f=b=c=d=e=h=l=a1=f1=b1=c1=d1=e1=h1=l1=i=iff1=iff2=im=r=0;
      ixoriy=new_ixoriy=0;
      ix=iy=sp=pc=0;
      tstates=radjust=0;
      nextlinetime=0;
      vsyncpend=vsynclen=0;
      hsyncskip=0;


	FILE *fp;
	
	fprintf(stdout,"Load Z81 FILE:%s\n",filename);

	if ((fp = fopen(filename, "r")) == NULL) {
		fprintf(stderr, "Cannot read from %s\n", filename);
		return;
	}


	/* Read the 'Z81' one line at a time (trailing [CR]LFs are read too) */
	index = -1;
	line_count = 0;
	while ((fgets(line, 256, fp)) != NULL) {

		line_count++;
		/* Remove the trailing [CR]LFs */
		for (count = strlen(line) - 1; count >= 0; count--)
			if (line[count] == 13 || line[count] == 10) line[count] = 0;
		/* Process the line ignoring comments */
		if (strlen(line) && line[0] != '#') {
 

                   
/***************************************************************************                         
 Init Z80 CPU Registers.

> [CPU]                     
> PC 0DA6   SP  7FD6
//   pc=0x207;
//   sp=0x7FFC;
> HL 4CC0   HL_ 0000
// h=0x4C;
// l=0xC0;
// h1=0x00;
// l1=0x00;
> DE 4CBF   DE_ 212B
//  d=0x4C;
//  e=0xBF; 
//  d1=0x21;
//  e1=0x2B;
> BC 0076   BC_ 0000
// b=0x00;
// c=0x66;
// b1=0x00;
// c1=0x00;
> AF 1DA3   AF_ 4A89
// a=0x1D;
// f=0xA3;
// a1=0x4A;
// f1=0x89;
> IX 0281   IY  4000 
// ix=0x281;
// iy=0x4000;
>  IR 1EDD
// i=0x1E;
// r=0xDD;
>  IM 00     IF1 01
// im=0;
// iff1=1;
>  HT 00     IF2 01
// HT ???
//iff2=1;

> [ZX81]
> NMI 01    HSYNC 01
//nmigen=1;
//hsyncskip=1;

> ROW 007  
//vsynclen=7;

 // radjust=0xCA; ???
 ***************************************************************************/


			strcpy(key, "PC ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&pc);
				strncpy(line,&line[10],8);
			}
			strcpy(key, "SP  ");
         	if (!strncmp(line, key, strlen(key))) {
			   sscanf(&line[strlen(key)], "%4x", &sp);
			}
			strcpy(key, "HL ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&HL);
                l=HL & 255;
                h=HL>>8;
                strncpy(line,&line[10],8);
			}
			strcpy(key, "HL_");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&HL1);
                l1=HL1 & 255;
				h1=HL1>>8;
            }
			strcpy(key, "DE ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&DE);
                e=DE & 255;
                d=DE>>8;
                strncpy(line,&line[10],8);
			}
			strcpy(key, "DE_");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&DE1);
                e1=DE1 & 255;
                d1=DE1>>8;
			}
			strcpy(key, "BC ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&BC);
                c=BC & 255;
				b=BC>>8;
				strncpy(line,&line[10],8);				
			}
			strcpy(key, "BC_");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&BC1);
                c1=BC1 & 255;
				b1=BC1>>8;
			}
			strcpy(key, "AF ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&AF);
                f=AF & 255;
				a=AF>>8;
				strncpy(line,&line[10],8);
			}
			strcpy(key, "AF_");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&AF1);
                f1=AF1 & 255;
				a1=AF1>>8;
			}
			strcpy(key, "IX ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&ix);
				strncpy(line,&line[10],8);				
			}
			strcpy(key, "IY ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&iy);
			}
			strcpy(key, "IR ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&IR);
                r=IR & 255;
				i=IR>>8;
			}
			strcpy(key, "IM ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%2x",&im);
				strncpy(line,&line[10],8);				
			}
			strcpy(key, "IF1 ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%2x",&iff1);
				iff1=iff1?1:0;
				interrupted=iff1;
			}
			strcpy(key, "HT ");
			if (!strncmp(line, key, strlen(key))) {
			//	sscanf(&line[strlen(key)], "%2x",&????);
				strncpy(line,&line[10],8);			
			}
			strcpy(key, "IF2 ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%2x",&iff2);
				iff2=iff2?1:0;
				if (iff2)iff2=iff1;
			}
			strcpy(key, "NMI ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%2x",&nmigen);
				nmigen=nmigen;
				strncpy(line,&line[10],8);				
			}
			strcpy(key, "HSYNC ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%2x",&hsyncskip);
				hsyncskip=hsyncskip?1:0;
			}
			strcpy(key, "ROW ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%x",&vsynclen);
			}
			strcpy(key, "MEMRANGE ");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%4x",&memorylow);
				fgets(datas,65536, fp);
				break;
			}
			
			
// auto-config. devices.
// Sound card id. = 0 , 1 [QuickSilva] or 2 [Zon-X] 

 			strcpy(key, "SCARD=");
			if (!strncmp(line, key, strlen(key))) {
				sscanf(&line[strlen(key)], "%x",&sound_ay_type);
				if (sound_ay_type) {
                 sound==1;
                 sound_ay=1;
                 sound_ay_init();
                 sound_init();
                 sound_ay_reset(); 
		         }
		         fprintf(stderr,"Sound Card:%i\n",sound_ay_type);
            }


		}
	}

/*
sp=0x;
h1=0;
l1=0;
d1=0x;
e1=0x;
b1=0;
c1=0;
a1=0x;
f1=0x;
iy=0x;
iff1=0;
iff2=0;
hsyncskip=1;
new_ixoriy=ix | iy;
ixoriy=new_ixoriy;
tstates=0;
radjust=0;
*/
 long memoffset;
 static char datamem;
     long repval;
     long datacnt;
     long memcnt;
            
    datacnt=0;      
    datamem=0;
    memoffset=memorylow;
                 
  while(datacnt<=strlen(datas))
 {

 if (datas[datacnt]>0x20)
 {      
        if (datas[datacnt]=='*')
        {
        datacnt++;//*                                     
        sscanf(&datas[datacnt], "%4x",&repval);
        datacnt+=5;//0000_ 
        sscanf(&datas[datacnt], "%2x",&datamem);
        datacnt+=2;//00
       
        //Fill memory with 'datamem' value.         
        for (memcnt=0;memcnt<repval;memcnt++)
        {
        mem[memoffset]=datamem;
        memoffset++;
        }
         }else{
        sscanf(&datas[datacnt], "%2x",&datamem);
        mem[memoffset]=datamem;
        memoffset++;       
        datacnt+=2;//00
        }
}
        datacnt++;//_ next char. bloc.  
 }   

fclose(fp);




// debug ****************************************************
//FILE *ms;    
//ms = fopen("memorysnap.bin","wb");
//for(memcnt=0;memcnt<=0xFFFF;memcnt++) fputc(mem[memcnt],ms);
//fclose(ms);

/*
	fprintf(stderr,"PC : %04x\n",pc);
	fprintf(stderr,"SP : %04x\n",sp);
	fprintf(stderr,"HL : %02x%02x\n",h,l);
	fprintf(stderr,"HL': %02x%02x\n",h1,l1);
	fprintf(stderr,"DE : %02x%02x\n",d,e);
	fprintf(stderr,"DE': %02x%02x\n",d1,e1);
	fprintf(stderr,"BC : %02x%02x\n",b,c);
	fprintf(stderr,"BC': %02x%02x\n",b1,c1);
	fprintf(stderr,"AF : %02x%02x\n",a,f);
	fprintf(stderr,"AF': %02x%02x\n",a1,f1);
	fprintf(stderr,"IX : %04x\n",ix);
	fprintf(stderr,"IY : %04x\n",iy);
    fprintf(stderr,"IR : %02x%02x\n",i,r);
    fprintf(stderr,"IM : %02x\n",im); // 0 ? , 1 and 2 !
    fprintf(stderr,"IF1: %02x\n",iff1); // 0
    fprintf(stderr,"IF2: %02x\n",iff2); // 0
    fprintf(stderr,"NMI: %02x\n",nmigen); // 0 or 1
    fprintf(stderr,"SYN: %02x\n",hsyncskip); // 0 or 1
    fprintf(stderr,"ROW: %03x\n",vsynclen); // 0 to 10.
    fprintf(stderr,"MEMORY LOW: %04x\n",memorylow); 
// debug ****************************************************/

interrupted=1;
framewait=1;
refresh_screen=1;
update_scrn();
	
}



/***************************************************************************
 * Save a Z81 File                                                         *
 ***************************************************************************/

void *Save_Z81(char *filename) {
     static unsigned long memoryrang;
     memoryrang=0x2000;
     
 FILE *fp;
 FILE *fp2;  
 
 /*==========================================================
 	fprintf(stderr,"PC : %04x\n",pc);
	fprintf(stderr,"SP : %04x\n",sp);
	fprintf(stderr,"HL : %02x%02x\n",h,l);
	fprintf(stderr,"HL': %02x%02x\n",h1,l1);
	fprintf(stderr,"DE : %02x%02x\n",d,e);
	fprintf(stderr,"DE': %02x%02x\n",d1,e1);
	fprintf(stderr,"BC : %02x%02x\n",b,c);
	fprintf(stderr,"BC': %02x%02x\n",b1,c1);
	fprintf(stderr,"AF : %02x%02x\n",a,f);
	fprintf(stderr,"AF': %02x%02x\n",a1,f1);
	fprintf(stderr,"IX : %04x\n",ix);
	fprintf(stderr,"IY : %04x\n",iy);
    fprintf(stderr,"IR : %02x%02x\n",i,r);
    fprintf(stderr,"IM : %04x\n",im);
    fprintf(stderr,"IF1: %02x\n",iff1);
    fprintf(stderr,"IF2: %02x\n",iff2);

 //==========================================================*/
 
 
 	fprintf(stdout,"Save Z81 FILE:%s\n",filename);

	if ((fp = fopen(filename, "w")) == NULL) {
		fprintf(stderr, "Cannot write to %s\n", filename);
		return;
	}

fprintf(fp,"[CPU]\n");
fprintf(fp,"PC %04x   SP  %04x\n",pc,sp);
fprintf(fp,"HL %02x%02x   HL_ %02x%02x\n",h,l,h1,l1);
fprintf(fp,"DE %02x%02x   DE_ %02x%02x\n",d,e,d1,e1);
fprintf(fp,"BC %02x%02x   BC_ %02x%02x\n",b,c,b1,c1);
fprintf(fp,"AF %02x%02x   AF_ %02x%02x\n",a,f,a1,f1);
fprintf(fp,"IX %04x   IY  %04x\n",ix,iy);
fprintf(fp,"IR %02x%02x\n",i,r);
fprintf(fp,"IM %02x     IF1 %02x\n",im?1:0,iff1?1:0);
fprintf(fp,"HT 00     IF2 %02x\n\n",iff2?1:0);
fprintf(fp,"[ZX81]\n");
fprintf(fp,"NMI %02x    HSYNC %02x\n",nmigen?2:0,hsyncskip?0:1);
fprintf(fp,"ROW %03x\n\n",vsynclen);
fprintf(fp,"[CONFIG]\nGCARD=%02x\n\n",sound_ay_type);
fprintf(fp,"[MEMORY]\nMEMRANGE %04x FFFF\n",memoryrang);
    
 static  unsigned long memcnt;
 static  long memiter;
 static unsigned char lastbyte='\0';
 
memiter=0;
lastbyte=mem[memcnt];
memcnt=memoryrang;

    
while (memcnt<=0xFFFF)
{
  //fputc(lastbyte,fp2);

  memcnt++;
  
  // compare [n] & [n+1] [n]=last value. [n+1]=next value.
 if (mem[memcnt]==lastbyte)
 {
// [n]=[n+1]
 // count how many bytes are chained.
 memiter++;
 lastbyte=mem[memcnt];   
 }
 else
 {
// [n]<>[n+1]
// Write the result in the file.
      if (memiter>0) // 
      {
      // Add a "*" tag.
      memiter++; // due to 0=1 byte, 1=2 bytes etc...
      fprintf(fp,"*%04x %02x ",memiter,lastbyte);
      memiter=0;
      //   memcnt--;
      }
      else
      {
      // Just one byte!
      fprintf(fp,"%02x ",lastbyte);
      } 
      lastbyte=mem[memcnt];                         
 }
}
      fprintf(fp,"\n[EOF]");
        
fclose(fp);     
}

/***************************************************************************
 * Get Z81 File window                                                     *
 ***************************************************************************
 True= Save the Z81 file.
 False= Load the Z81 file.
 */

BOOL Ask_Z81(BOOL bSave) {
   OPENFILENAME ofn;
   HWND hwnd;
   
      hwnd = NULL;
      
   char szFileName[MAX_PATH];

   ZeroMemory(&ofn, sizeof(ofn));
   szFileName[0] = 0;

   ofn.lStructSize = sizeof(ofn);
   ofn.hwndOwner = hwnd;
   ofn.lpstrFilter = "Z81 memory snap Files (*.Z81)\0*.Z81\0All Files (*.*)\0*.*\0\0";
   ofn.lpstrFile = szFileName;
   ofn.nMaxFile = MAX_PATH;
   ofn.lpstrDefExt = "z81";

   if(bSave)
   {
      ofn.Flags = OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY |
         OFN_OVERWRITEPROMPT;
         
      if(GetSaveFileName(&ofn))
      {
       Save_Z81(szFileName);     
       }                   
/*         if(!Save_Z81(szFileName))
         {
            MessageBox(hwnd, "Save file failed.", "Error",
               MB_OK | MB_ICONEXCLAMATION);
            return FALSE;
         }
 */        
  
   }     
   else
   {
      ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
      if(GetOpenFileName(&ofn))
      {
       Load_Z81(szFileName);
      }
      /*
         if(!Load_Z81(szFileName))
         {
            MessageBox(hwnd, "Load of file failed.", "Error",
               MB_OK | MB_ICONEXCLAMATION);
            return FALSE;
         }
*/

   }
   return TRUE;
}     
